#include <OneWire.h>
#include <DallasTemperature.h>
#include <Wire.h>
#include <MPU6050.h>
#include <TheThingsNetwork.h>

#define loraSerial Serial1
#define debugSerial Serial
#define freqPlan TTN_FP_EU868
#define ONE_WIRE_BUS 5
MPU6050 mpu;

// Set your AppEUI and AppKey
const char *appEui = "0000000000000000";
const char *appKey = "563C2795E5A15E1321E2AB33B331F0CE";

OneWire oneWire(ONE_WIRE_BUS);
TheThingsNetwork ttn(loraSerial, debugSerial, freqPlan);
DallasTemperature sensors(&oneWire);

uint16_t temp=0;
uint16_t turb=0;

void setup(void){
  Serial.begin(9600);
  sensors.begin();
  loraSerial.begin(57600);
  debugSerial.begin(9600);

  // Wait a maximum of 10s for Serial Monitor
  while (!debugSerial && millis() < 10000)
    ;

  debugSerial.println("-- STATUS");
  ttn.showStatus();

  debugSerial.println("-- JOIN");
  ttn.join(appEui, appKey);
  
  // Initialize MPU6050
  Serial.println("Initialize MPU6050");
  while(!mpu.begin(MPU6050_SCALE_2000DPS, MPU6050_RANGE_2G))
  {
    Serial.println("Could not find a valid MPU6050 sensor, check wiring!");
    delay(500);
  }
    mpu.calibrateGyro();
    mpu.setThreshold(3);
    checkSettings();
}

void checkSettings()
{
  Serial.println();
  
  Serial.print(" * Sleep Mode:        ");
  Serial.println(mpu.getSleepEnabled() ? "Enabled" : "Disabled");
  
  Serial.print(" * Clock Source:      ");
  switch(mpu.getClockSource())
  {
    case MPU6050_CLOCK_KEEP_RESET:     Serial.println("Stops the clock and keeps the timing generator in reset"); break;
    case MPU6050_CLOCK_EXTERNAL_19MHZ: Serial.println("PLL with external 19.2MHz reference"); break;
    case MPU6050_CLOCK_EXTERNAL_32KHZ: Serial.println("PLL with external 32.768kHz reference"); break;
    case MPU6050_CLOCK_PLL_ZGYRO:      Serial.println("PLL with Z axis gyroscope reference"); break;
    case MPU6050_CLOCK_PLL_YGYRO:      Serial.println("PLL with Y axis gyroscope reference"); break;
    case MPU6050_CLOCK_PLL_XGYRO:      Serial.println("PLL with X axis gyroscope reference"); break;
    case MPU6050_CLOCK_INTERNAL_8MHZ:  Serial.println("Internal 8MHz oscillator"); break;
  }
  
  Serial.print(" * Gyroscope:         ");
  switch(mpu.getScale())
  {
    case MPU6050_SCALE_2000DPS:        Serial.println("2000 dps"); break;
    case MPU6050_SCALE_1000DPS:        Serial.println("1000 dps"); break;
    case MPU6050_SCALE_500DPS:         Serial.println("500 dps"); break;
    case MPU6050_SCALE_250DPS:         Serial.println("250 dps"); break;
  } 
  
  Serial.print(" * Gyroscope offsets: ");
  Serial.print(mpu.getGyroOffsetX());
  Serial.print(" / ");
  Serial.print(mpu.getGyroOffsetY());
  Serial.print(" / ");
  Serial.println(mpu.getGyroOffsetZ());
  
  Serial.println();
}

void loop(void){ 
  sensors.requestTemperatures(); 
  temp=sensors.getTempCByIndex(0)*100;
  int sensorValue = analogRead(A5);

  Serial.println("---------------------------------");
  Serial.println("Mediciones");
  Serial.println("---------------------------------");

  Serial.print("Nivel de turbidez: ");
  if(sensorValue < 800){
    Serial.println("Agua turbia");
    turb=0;
    
  }   
  if(sensorValue > 800){
    Serial.println("Agua limpia");
    turb=1;
  }
  Serial.print("Temperatura del agua: ");
  Serial.print(temp);
  Serial.println(" Cº");

  Vector rawGyro = mpu.readRawGyro();
  Vector normGyro = mpu.readNormalizeGyro();
  uint16_t gyroX=normGyro.XAxis; 
  uint16_t gyroY=normGyro.YAxis; 
  uint16_t gyroZ=normGyro.ZAxis; 

  Serial.println("Movimiento de la boya:");
  Serial.print("Xraw = ");
  Serial.print(rawGyro.XAxis);
  Serial.print(" Yraw = ");
  Serial.print(rawGyro.YAxis);
  Serial.print(" Zraw = ");
  Serial.println(rawGyro.ZAxis);

  Serial.print("Xnorm = ");
  Serial.print(normGyro.XAxis);
  Serial.print(" Ynorm = ");
  Serial.print(normGyro.YAxis);
  Serial.print(" Znorm = ");
  Serial.println(normGyro.ZAxis);

  Serial.println();
  // Prepare payload of 1 byte to indicate LED status
  byte payload[10];
  payload[0] = highByte(turb);
  payload[1] = lowByte(turb);
  payload[2] = highByte(temp);
  payload[3] = lowByte(temp);
  payload[4] = highByte(gyroX);
  payload[5] = lowByte(gyroX);
  payload[6] = highByte(gyroY);
  payload[7] = lowByte(gyroY);
  payload[8] = highByte(gyroZ);
  payload[9] = lowByte(gyroZ);
  
  // Send it off
  ttn.sendBytes(payload, sizeof(payload));

  delay(1000);
}
